/*
 * CKEditor - The text editor for Internet - http://ckeditor.com
 * Copyright (C) 2003-2008 Frederico Caldeira Knabben
 *
 * == BEGIN LICENSE ==
 *
 * Licensed under the terms of any of the following licenses at your
 * choice:
 *
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 *
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 *
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 *
 * == END LICENSE ==
 */

/**
 * @fileoverview The "fakeobjects" plugin. It handles placeholder DOM nodes
 * 	in WYSIWYG mode.
 */

(function()
{
	CKEDITOR.plugins.add( 'fakeobjects',
		{
			init : function( editor, pluginPath )
			{
				var flashExtensionRegex = /\.swf($|#|\?)/i,
					objectTypes =
					[
						{
							match : function( nodeName, attributes )
							{
								return nodeName == 'embed' && ( attributes.type == 'application/x-shockwave-flash' || flashExtensionRegex.test( attributes.src || '' ) );
							},
							cssClass : 'flash',
							priority : 10
						},
						{
							match : function( nodeName, attributes )
							{
								return nodeName == 'embed' || nodeName == 'object';
							},
							cssClass : 'object',
							priority : 0x100000000
						}
					],
					makeTagOpenerHtml = function( tagName, attributes )
					{
						var attribStr = [], html = [ '<' + tagName ];
						for ( var i in attributes )
							attribStr.push( i + '="' + CKEDITOR.tools.htmlEncode( attributes[i] ) + '"');
						if ( attribStr.length > 0 )
							html.push( ' ', attribStr.join( ' ' ) );
						html.push( '>' );
						return html.join( '' );
					},
					protectedHtmlRegex = /<img(?=\s).*?\s_cke_protected_html=((?:(?:\s*)("|').*?\2)|(?:[^"'][^ >]+))(?:"[^"]*"|'[^']*'|[^<])*>(?:<\/img>)?/gi,
					protectHtmlRestore_ReplaceTags = function( htmlTag, s1 )
					{
						return decodeURIComponent( s1.substr( 1, s1.length - 2 ) );
					},
					cssWidthRegex = /width\s*:\s*([0-9]+)\s*(?:px)?/i,
					cssHeightRegex = /height\s*:\s*([0-9]+)\s*(?:px)?/i;
				
					/**
					 * Manages element placeholders in WYSIWYG area.
					 * @namespace
					 * @example
					 */
					CKEDITOR.plugins.fakeobjects =
					{
						/**
						 * Converts an element into a placeholder &lt;img&gt; element, for adding
						 * into the WYSIWYG area.
						 * @param {CKEDITOR.dom.element} element Input DOM element.
						 * @returns {CKEDITOR.dom.element} The placeholder &lt;img&gt; element.
						 * @example
						 */
						protectElement : function( element )
						{
							var $ = element.$.cloneNode( true ),
								doc = $.ownerDocument,
								temp = doc.createElement( 'div' ),
								html;

							// Get the object's HTML code.
							temp.appendChild( $ );
							html = temp.innerHTML;

							// Get the fake element's CSS class.
							var cssClass = 'unknown';
							for ( var i = 0 ; i < objectTypes.length ; i++ )
							{
								if ( objectTypes[i].match( element.getName(), element.$ ) )
								{
									cssClass = '_cke_fakeobject' + ' ' + objectTypes[i].cssClass;
									break;
								}
							}

							// Make the fake element.
							var fakeRawElement = doc.createElement( 'img' );
							img.className = cssClass;
							img.src = CKEDITOR.getUrl( 'images/spacer.gif' ); 
							img.style.cssText = $.style.cssText;
							return new CKEDITOR.dom.element( fakeRawElement );
						},

						/**
						 * Converts a placeholder &lt;img&gt; element back to the real element.
						 * @param {CKEDITOR.dom.element} fakeImgElement The placeholder &lt;img&gt;.
						 * @returns {CKEDITOR.dom.element} The real element.
						 * @example
						 */
						restoreElement : function( fakeImgElement )
						{
							var html = decodeURIComponent( fakeImgElement.getAttribute( '_cke_protected_html' ) ),
								realElement = CKEDITOR.dom.element.createFromHtml( html, editor.document );

							return realElement;
						},

						/**
						 * Converts protectable elements in an HTML string to placeholders.
						 * @param {String} html HTML with protectable elements.
						 * @returns {String} HTML with placeholders.
						 * @example
						 */
						protectHtml : function( html )
						{
							var parser = new CKEDITOR.htmlParser(),
								tagDepth = 0, processedHtml = [],
								protectedHtml = [], inProtection = false;

							parser.onTagOpen = function( tagName, attributes )
							{
								if ( inProtection )
								{
									protectedHtml.push( makeTagOpenerHtml( tagName, attributes ) );
									return;
								}

								for ( var i = 0 ; i < objectTypes.length ; i++ )
								{
									if ( objectTypes[i].match( tagName, attributes ) )
									{
										inProtection = true;
										tagDepth = 1;

										// Get the original object's width and height.
										var styles = attributes.style || '',
											widthMatch = styles.match( cssWidthRegex ) || [ 80, 80 ],
											heightMatch = styles.match( cssHeightRegex ) || [ 80, 80 ];

										// Create the fake <img> tag.
										processedHtml.push( '<img src="',
											CKEDITOR.getUrl( 'images/spacer.gif' ),
											'" ',
											'class="_cke_fakeobject ' + objectTypes[i].cssClass + '" ',
											'style="width:' + widthMatch[1] + 'px;height:' + heightMatch[1] + 'px" ',
											'_cke_protected_html="');
										arguments.callee.call( this, tagName, attributes );
										return;
									}
								}

								processedHtml.push( makeTagOpenerHtml( tagName, attributes ) );
							};

							parser.onText = function( text )
							{
								inProtection ? protectedHtml.push( text ) : processedHtml.push( text );
							};

							parser.onComment = function( comment )
							{
								inProtection ? protectedHtml.push( '<!--' + comment + '-->' ) : processedHtml.push( '<!--' + comment + '-->' );
							};

							parser.onTagClose = function( tagName )
							{
								if ( inProtection )
								{
									tagDepth--;
									protectedHtml.push( '</' + tagName + '>' );
									if ( tagDepth < 1 )
									{
										inProtection = false;
										processedHtml.push( encodeURIComponent( protectedHtml.join( '' ) ), '" />' );
									}
								}
								else
									processedHtml.push( '</' + tagName + '>' );
							};

							parser.parse( html );
							return processedHtml.join( '' );
						},

						/**
						 * Restores placeholders in an HTML string back to their original elements.
						 * @param {String} html HTML with placeholders.
						 * @returns {String} Restored HTML.
						 * @example
						 */
						restoreHtml : function( html )
						{
							return html.replace( protectedHtmlRegex, protectHtmlRestore_ReplaceTags );
						},

						/**
						 * Adds an object type to be displayed by placeholders.
						 * @param {Function} matchFunc A predicate function to determine whether
						 * an object needs to be replaced by placeholders or not. The function
						 * should have the following signature:
						 * <blockquote>function( tagName, attributes )</blockquote>
						 * In which tagName is the object's tag name, and attributes is an object
						 * storing all the object's attributes.
						 * @param {String} cssClass The CSS class that should be added to the
						 * placeholder &lt;img&gt; for representing this type of object.
						 * @param {Number} priority (Optional) An integer representing the
						 * priority of this type of object. If an element matches the descriptions
						 * of two object types, the object type of <strong>lower</strong> priority
						 * takes precedance.
						 * @example
						 */
						addObjectType : function( matchFunc, cssClass, priority )
						{
							if ( priority === undefined )
								priority = 10;

							var obj = {
								match : matchFunc,
								cssClass : cssClass,
								priority : priority
							};

							for ( var i = objectTypes.length - 1 ; i >= 0 ; i-- )
							{
								if ( objectTypes[i].priority < priority )
								{
									objectTypes.splice( i + 1, 0, obj );
									return;
								}
							}

							objectTypes.unshift( obj );
						}
					};
			}
		} );
})();
